package javax.swing.plaf.metal;

import javax.swing.Icon;
import java.awt.Insets;
import java.awt.FontMetrics;
import java.awt.Font;
import java.awt.Dimension;
import javax.swing.ButtonModel;
import javax.swing.text.View;
import javax.swing.SwingUtilities;
import java.awt.Rectangle;
import sun.swing.SwingUtilities2;
import java.awt.Graphics;
import javax.swing.LookAndFeel;
import javax.swing.UIManager;
import javax.swing.AbstractButton;
import sun.awt.AppContext;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import java.awt.Color;
import javax.swing.plaf.basic.BasicRadioButtonUI;

public class MetalRadioButtonUI extends BasicRadioButtonUI
{
    private static final Object METAL_RADIO_BUTTON_UI_KEY;
    protected Color focusColor;
    protected Color selectColor;
    protected Color disabledTextColor;
    private boolean defaults_initialized;
    
    public MetalRadioButtonUI() {
        this.defaults_initialized = false;
    }
    
    public static ComponentUI createUI(JComponent component) {
        AppContext appContext = AppContext.getAppContext();
        MetalRadioButtonUI value = (MetalRadioButtonUI)appContext.get(MetalRadioButtonUI.METAL_RADIO_BUTTON_UI_KEY);
        if (value == null) {
            value = new MetalRadioButtonUI();
            appContext.put(MetalRadioButtonUI.METAL_RADIO_BUTTON_UI_KEY, value);
            return value;
        }
        return value;
    }
    
    public void installDefaults(AbstractButton abstractButton) {
        super.installDefaults(abstractButton);
        if (!this.defaults_initialized) {
            this.focusColor = UIManager.getColor(this.getPropertyPrefix() + "focus");
            this.selectColor = UIManager.getColor(this.getPropertyPrefix() + "select");
            this.disabledTextColor = UIManager.getColor(this.getPropertyPrefix() + "disabledText");
            this.defaults_initialized = true;
        }
        LookAndFeel.installProperty(abstractButton, "opaque", Boolean.TRUE);
    }
    
    @Override
    protected void uninstallDefaults(AbstractButton b) {
        super.uninstallDefaults(b);
        this.defaults_initialized = false;
    }
    
    protected Color getSelectColor() {
        return this.selectColor;
    }
    
    protected Color getDisabledTextColor() {
        return this.disabledTextColor;
    }
    
    protected Color getFocusColor() {
        return this.focusColor;
    }
    
    @Override
    public synchronized void paint(Graphics graphics, JComponent c) {
        AbstractButton abstractButton = (AbstractButton)c;
        ButtonModel model = abstractButton.getModel();
        Dimension size = c.getSize();
        int width = size.width;
        int height = size.height;
        Font font = c.getFont();
        graphics.setFont(font);
        FontMetrics fontMetrics = SwingUtilities2.getFontMetrics(c, graphics, font);
        Rectangle viewR = new Rectangle(size);
        Rectangle iconR = new Rectangle();
        Rectangle textR = new Rectangle();
        Insets insets = c.getInsets();
        viewR.x += insets.left;
        viewR.y += insets.top;
        viewR.width -= insets.right + viewR.x;
        viewR.height -= insets.bottom + viewR.y;
        Icon icon = abstractButton.getIcon();
        String layoutCompoundLabel = SwingUtilities.layoutCompoundLabel(c, fontMetrics, abstractButton.getText(), (icon != null) ? icon : this.getDefaultIcon(), abstractButton.getVerticalAlignment(), abstractButton.getHorizontalAlignment(), abstractButton.getVerticalTextPosition(), abstractButton.getHorizontalTextPosition(), viewR, iconR, textR, abstractButton.getIconTextGap());
        if (c.isOpaque()) {
            graphics.setColor(abstractButton.getBackground());
            graphics.fillRect(0, 0, size.width, size.height);
        }
        if (icon != null) {
            if (!model.isEnabled()) {
                if (model.isSelected()) {
                    icon = abstractButton.getDisabledSelectedIcon();
                }
                else {
                    icon = abstractButton.getDisabledIcon();
                }
            }
            else if (model.isPressed() && model.isArmed()) {
                icon = abstractButton.getPressedIcon();
                if (icon == null) {
                    icon = abstractButton.getSelectedIcon();
                }
            }
            else if (model.isSelected()) {
                if (abstractButton.isRolloverEnabled() && model.isRollover()) {
                    icon = abstractButton.getRolloverSelectedIcon();
                    if (icon == null) {
                        icon = abstractButton.getSelectedIcon();
                    }
                }
                else {
                    icon = abstractButton.getSelectedIcon();
                }
            }
            else if (abstractButton.isRolloverEnabled() && model.isRollover()) {
                icon = abstractButton.getRolloverIcon();
            }
            if (icon == null) {
                icon = abstractButton.getIcon();
            }
            icon.paintIcon(c, graphics, iconR.x, iconR.y);
        }
        else {
            this.getDefaultIcon().paintIcon(c, graphics, iconR.x, iconR.y);
        }
        if (layoutCompoundLabel != null) {
            View view = (View)c.getClientProperty("html");
            if (view != null) {
                view.paint(graphics, textR);
            }
            else {
                int displayedMnemonicIndex = abstractButton.getDisplayedMnemonicIndex();
                if (model.isEnabled()) {
                    graphics.setColor(abstractButton.getForeground());
                }
                else {
                    graphics.setColor(this.getDisabledTextColor());
                }
                SwingUtilities2.drawStringUnderlineCharAt(c, graphics, layoutCompoundLabel, displayedMnemonicIndex, textR.x, textR.y + fontMetrics.getAscent());
            }
            if (abstractButton.hasFocus() && abstractButton.isFocusPainted() && textR.width > 0 && textR.height > 0) {
                this.paintFocus(graphics, textR, size);
            }
        }
    }
    
    @Override
    protected void paintFocus(Graphics graphics, Rectangle rectangle, Dimension dimension) {
        graphics.setColor(this.getFocusColor());
        graphics.drawRect(rectangle.x - 1, rectangle.y - 1, rectangle.width + 1, rectangle.height + 1);
    }
    
    static {
        METAL_RADIO_BUTTON_UI_KEY = new Object();
    }
}
